
/*******************************************************************                   
                      2-D REDUNDANT WAVELET TRANSFORM

calculated with quadratic spline, according to Mallat and Zhong.
It is a redundant (nondownsampled) transform.

Aleksandra Pizurica/TELIN-RUG
********************************************************************/ 
   
  
#include "mex.h"
#include <math.h>

void wt1_mz(double *a, double *h, double *c, int p, int N, int D);


void workFcn(double *X, double *A, double *D, int Scale, int N)
{  
 
int i,j;
double *Row_In, *Row_Out;
double Detail_Filt[5]={0,2,-2,0,0};
double Lowpass_Filt[5]={0.125,0.375,0.375,0.125,0};
double Detail_Filt_Length=5, Lowpass_Filt_Length=5;

Row_In=malloc(N*sizeof(double)); 
Row_Out=malloc(N*sizeof(double)); 

for (i=0; i<N; i++)
     Row_In[i]=X[i];

   wt1_mz(Row_In, Detail_Filt, Row_Out, Scale, N, Detail_Filt_Length);
   for(i=0; i<N; i++)
      D[i]=Row_Out[i];

   wt1_mz(Row_In, Lowpass_Filt, Row_Out, Scale, N, Detail_Filt_Length);
   for(i=0; i<N; i++)
      A[i]=Row_Out[i];


free(Row_In);
free(Row_Out);

} /* End of workFcn */


/*---------------------------------------------------------------------------------*/

void wt1_mz(double *a, double *h, double *c, int p, int N, int D)
{  
int i,j,Dext,I1,I2,len,t;
double *ap,*he;

t=pow(2,p);
ap=malloc(3*N*sizeof(double));
Dext=(D-1)*t+1;
he=malloc(Dext*sizeof(double));

/* Solving the border problems: Input vector should be made periodical and mirrored*/

 for (i=0; i<N; i++)
   {
     ap[i+N]=a[i];
     ap[3*N-1-i]=a[i];
     ap[N-1-i]=a[i];
   }

/* Extending the impulse response h: Putting 2^p-1 zeros between the coefficients of h*/

 for(i=0; i<Dext; i++)
    he[i]=0;
 for(i=0; i<D; i++)
    he[i*t]=h[i];

/* Convolution: c=conv(ap,he) */

 I2=((D-1)/2)*t;  /*index of the element h(0) in the vector he*/

 for(j=0; j<N; j++){
   c[j]=0;
   I1=N+j;          /*index of the element a(0+j) in the vector ap*/
   if ((I1+I2+1)>=Dext)
      len=Dext;
   else
      len=I1+I2+1;
   for(i=0; i<len; i++){
      c[j]+=he[i]*ap[I1+I2-i];
   }
 }
   

free(ap);
free(he);
       
}  /* End of wt1_mz*/


/*----------------------------------------------------------------------------------*/

/*Defining the gateway function, i.e. the mex function*/

#define ARRAY_IN prhs[0]
#define SCALE prhs[1]
#define LOWPASS plhs[0]
#define DETAIL plhs[1]
 
void mexFunction(
               int nlhs,
               mxArray  *plhs[],
               int     nrhs,
               const mxArray  *prhs[]
               )
{
  int row, col, i, N;
  int Scale;
  double *ArrayIn, *Lowpass, *Detail;

  if (nlhs!=2)
      mexErrMsgTxt("program requires two output arguments [LOWPASS DETAIL].");
  if (nrhs!=2)
      mexErrMsgTxt("program requires two input arguments [ARRAY_IN SCALE]");
  

  N=mxGetN(ARRAY_IN);
  
  Scale=mxGetScalar(SCALE); 
  
  /* Allocate memory for return arrays */
  LOWPASS = mxCreateDoubleMatrix(1, N, mxREAL);  
  Lowpass = mxGetPr(LOWPASS);
  
  DETAIL = mxCreateDoubleMatrix(1, N, mxREAL);  
  Detail = mxGetPr(DETAIL);

   /* Dynamic allocation of memory for ArrayIn */

  ArrayIn = malloc(N*sizeof(double));

  for(row=0;row<N;row++)
    ArrayIn[row] = (mxGetPr(ARRAY_IN))[row];
               

/*Call workFcn function*/ 
workFcn(ArrayIn,Lowpass,Detail,Scale,N);

free(ArrayIn);
 
}/*mexFcn*/